VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Hexer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit
'=====
'Hexer
'=====
'
'A VB6 class providing two methods intended to help make entry,
'use, and display of binary values easier.
'
'Several formats are implemented, set via the HexFormat property.
'
'This is a "PredeclaredId" class.  There is a global instance
'of this class named "Hexer" provided automatically.
'

Private Const CRYPT_STRING_HEX As Long = &H4&
Private Const CRYPT_STRING_HEXADDR As Long = &HA&
Private Const CRYPT_STRING_HEXASCIIADDR As Long = &HB&
Private Const CRYPT_STRING_HEXRAW As Long = &HC&

Private Declare Function CryptBinaryToString Lib "Crypt32" _
    Alias "CryptBinaryToStringW" ( _
    ByVal pbBinary As Long, _
    ByVal cbBinary As Long, _
    ByVal dwFlags As HexFormats, _
    ByVal pszString As Long, _
    ByRef pcchString As Long) As Long

Private Declare Function CryptStringToBinary Lib "Crypt32" _
    Alias "CryptStringToBinaryW" ( _
    ByVal pszString As Long, _
    ByVal cchString As Long, _
    ByVal dwFlags As HexFormats, _
    ByVal pbBinary As Long, _
    ByRef pcbBinary As Long, _
    ByRef pdwSkip As Long, _
    ByRef pdwFlags As Long) As Long
    
Public Enum HexFormats
    [_hfmtUseCurrent] = -1
    hfmtHex = CRYPT_STRING_HEX
    hfmtHexAddr = CRYPT_STRING_HEXADDR
    hfmtHexAsciiAddr = CRYPT_STRING_HEXASCIIADDR
    hfmtHexRaw = CRYPT_STRING_HEXRAW
    [_hfmtDefault] = hfmtHexAsciiAddr
End Enum
'Preserves case of enum value names.
#If False Then
Private hfmtHex, hfmtHexAddr, hfmtHexAsciiAddr, hfmtHexRaw
#End If

Public HexFormat As HexFormats
Attribute HexFormat.VB_VarDescription = "Hex format to convert to and from"

Public Function BinaryToHex( _
    ByRef Binary() As Byte, _
    Optional ByVal NewFormat As HexFormats = [_hfmtUseCurrent]) As String
    
    Dim lngInLen As Long
    Dim lngOutLen As Long
    
    If NewFormat <> [_hfmtUseCurrent] Then HexFormat = NewFormat
    lngInLen = UBound(Binary) - LBound(Binary) + 1
    
    'Determine output String length required.
    If CryptBinaryToString(VarPtr(Binary(LBound(Binary))), _
                           lngInLen, _
                           HexFormat, _
                           0&, _
                           lngOutLen) = 0 Then
        Err.Raise &H80044102, _
                  "Hexer.BinaryToHex", _
                  "CryptBinaryToString failed, error " & CStr(Err.LastDllError)
    Else
        'Convert binary.
        BinaryToHex = String(lngOutLen, 0)
        If CryptBinaryToString(VarPtr(Binary(LBound(Binary))), _
                               lngInLen, _
                               HexFormat, _
                               StrPtr(BinaryToHex), _
                               lngOutLen) = 0 Then
            Err.Raise &H80044102, _
                      "Hexer.BinaryToHex", _
                      "CryptBinaryToString failed, error " & CStr(Err.LastDllError)
        End If
        If HexFormat = hfmtHexRaw Then
            'Discard formatting we don't want here.
            BinaryToHex = Replace$(Replace$(BinaryToHex, " ", ""), vbNewLine, "")
        End If
    End If
End Function

Public Function HexToBinary( _
    ByRef HexString As String, _
    Optional ByVal NewFormat As HexFormats = [_hfmtUseCurrent]) As Byte()
Attribute HexToBinary.VB_Description = "Convert hex String to binary Byte array, optionally changing HexFormat"
    
    Dim lngOutLen As Long
    Dim dwActualUsed As Long
    Dim bytBinary() As Byte
    
    If NewFormat <> [_hfmtUseCurrent] Then HexFormat = NewFormat
    If Len(HexString) < 1 Then Exit Function
    
    'Determine output buffer length required.
    If CryptStringToBinary(StrPtr(HexString), _
                           Len(HexString), _
                           HexFormat, _
                           0&, _
                           lngOutLen, _
                           0&, _
                           dwActualUsed) = 0 Then
        Err.Raise &H80044100, _
                  "Hexer.HexToBinary", _
                  "CryptStringToBinary failed, error " & CStr(Err.LastDllError)
    Else
        'Convert to binary.
        ReDim bytBinary(lngOutLen - 1)
        If CryptStringToBinary(StrPtr(HexString), _
                               Len(HexString), _
                               HexFormat, _
                               VarPtr(bytBinary(0)), _
                               lngOutLen, _
                               0&, _
                               dwActualUsed) = 0 Then
            Err.Raise &H80044100, _
                      "Hexer.HexToBinary", _
                      "CryptStringToBinary failed, error " & CStr(Err.LastDllError)
        Else
            HexToBinary = bytBinary
        End If
    End If
End Function

Private Sub Class_Initialize()
    HexFormat = [_hfmtDefault]
End Sub
